@@ -7,7 +7,7 @@ import json
 from . import api as inventree_api
 
 
-INVENTREE_PYTHON_VERSION = "0.8.1"
+INVENTREE_PYTHON_VERSION = "0.9.0"
 
 
 logger = logging.getLogger('inventree')
@@ -510,3 +510,109 @@ class ImageMixin:
             return self._api.downloadFile(self.image, destination, **kwargs)
         else:
             raise ValueError(f"Part '{self.name}' does not have an associated image")
+
+
+class StatusMixin:
+    """Class adding functionality to assign a new status by calling
+    - complete
+    - cancel
+    on supported items.
+    
+    Other functions, such as
+    - ship
+    - finish
+    - issue
+    can be reached through _statusupdate function
+    """
+
+    def _statusupdate(self, status: str, reload=True, data=None, **kwargs):
+
+        # Check status
+        if status not in [
+            'complete',
+            'cancel',
+            'ship',
+            'issue',
+            'finish',
+        ]:
+            raise ValueError(f"Order stats {status} not supported.")
+
+        # Set the url
+        URL = self.URL + f"/{self.pk}/{status}"
+
+        if data is None:
+            data = {}
+
+        data.update(kwargs)
+
+        # Send data
+        response = self._api.post(URL, data)
+
+        # Reload
+        if reload:
+            self.reload()
+
+        # Return
+        return response
+
+    def complete(self, **kwargs):
+        
+        return self._statusupdate(status='complete', **kwargs)
+
+    def cancel(self, **kwargs):
+        
+        return self._statusupdate(status='cancel', **kwargs)
+
+
+class BarcodeMixin:
+    """Adds barcode scanning functionality to various data types.
+
+    Any class which inherits from this mixin can assign (or un-assign) barcode data.
+    """
+
+    @classmethod
+    def barcodeModelType(cls):
+        """Return the model type name required for barcode assignment.
+        
+        Default value is the lower-case class name ()
+        """
+        return cls.__name__.lower()
+
+    def assignBarcode(self, barcode_data: str, reload=True):
+        """Assign an arbitrary barcode to this object (in the database).
+        
+        Arguments:
+            barcode_data: A string containing arbitrary barcode data
+        """
+
+        model_type = self.barcodeModelType()
+
+        response = self._api.post(
+            '/barcode/link/',
+            {
+                'barcode': barcode_data,
+                model_type: self.pk,
+            }
+        )
+
+        if reload:
+            self.reload()
+
+        return response
+
+    def unassignBarcode(self, reload=True):
+        """Unassign a barcode from this object"""
+
+        model_type = self.barcodeModelType()
+
+        response = self._api.post(
+            '/barcode/unlink/',
+            {
+                model_type: self.pk,
+            }
+        )
+
+        if reload:
+            self.reload()
+
+        return response
