@@ -5,7 +5,11 @@ import inventree.part
 import inventree.company
 
 
-class PurchaseOrder(inventree.base.MetadataMixin, inventree.base.InventreeObject):
+class PurchaseOrder(
+    inventree.base.MetadataMixin,
+    inventree.base.InventreeObject,
+    inventree.base.StatusMixin
+):
     """ Class representing the PurchaseOrder database model """
 
     URL = 'order/po'
@@ -47,6 +51,14 @@ class PurchaseOrder(inventree.base.MetadataMixin, inventree.base.InventreeObject
             order=self.pk,
         )
 
+    def issue(self, **kwargs):
+        """
+        Issue the purchase order
+        """
+
+        # Return
+        return self._statusupdate(status='issue', **kwargs)
+
 
 class PurchaseOrderLineItem(inventree.base.InventreeObject):
     """ Class representing the PurchaseOrderLineItem database model """
@@ -92,7 +104,11 @@ class PurchaseOrderAttachment(inventree.base.Attachment):
     REQUIRED_KWARGS = ['order']
 
 
-class SalesOrder(inventree.base.MetadataMixin, inventree.base.InventreeObject):
+class SalesOrder(
+    inventree.base.MetadataMixin,
+    inventree.base.InventreeObject,
+    inventree.base.StatusMixin
+):
     """ Class respresenting the SalesOrder database model """
 
     URL = 'order/so'
@@ -186,9 +202,9 @@ class SalesOrderLineItem(inventree.base.InventreeObject):
         possibly because no stock items are available.
         """
 
-        # If stockitems are not defined, get the list
+        # If stockitems are not defined, get the list of available stock items
         if stockitems is None:
-            stockitems = self.getPart().getStockItems()
+            stockitems = self.getPart().getStockItems(include_variants=False, in_stock=True, available=True)
 
         # If no quantity is defined, calculate the number of required items
         # This is the number of sold items not yet allocated, but can not
@@ -209,6 +225,7 @@ class SalesOrderLineItem(inventree.base.InventreeObject):
         # Look through stock items, assign items until the required amount
         # is reached
         items = list()
+
         for SI in stockitems:
             
             # Check if we are done
@@ -256,7 +273,10 @@ class SalesOrderAttachment(inventree.base.Attachment):
     REQUIRED_KWARGS = ['order']
 
 
-class SalesOrderShipment(inventree.base.InventreeObject):
+class SalesOrderShipment(
+    inventree.base.InventreeObject,
+    inventree.base.StatusMixin
+):
     """Class representing a shipment for a SalesOrder"""
 
     URL = 'order/so/shipment'
@@ -304,16 +324,14 @@ class SalesOrderShipment(inventree.base.InventreeObject):
         shipment_date=None,
         tracking_number='',
         invoice_number='',
-        link=''
+        link='',
+        **kwargs
     ):
         """
         Complete the shipment, with given shipment_date, or reasonable
         defaults.
         """
 
-        # Customise URL
-        url = f'order/so/shipment/{self.pk}/ship'
-
         # Create data from given inputs
         data = {
             'shipment_date': shipment_date,
@@ -322,11 +340,9 @@ class SalesOrderShipment(inventree.base.InventreeObject):
             'link': link
         }
 
-        # Send data
-        response = self._api.post(url, data)
-
-        # Reload
-        self.reload()
-
         # Return
-        return response
+        return self._statusupdate(status='ship', data=data, **kwargs)
+
+    def ship(self, *args, **kwargs):
+        """Alias for complete function"""
+        self.complete(*args, **kwargs)
