@@ -6,13 +6,18 @@ import logging
 import inventree.api
 import inventree.base
 import inventree.part
+import inventree.label
 
 
-class StockLocation(inventree.base.MetadataMixin, inventree.base.InventreeObject):
+class StockLocation(inventree.base.BarcodeMixin, inventree.base.MetadataMixin, inventree.label.LabelPrintingMixing, inventree.base.InventreeObject):
     """ Class representing the StockLocation database model """
 
     URL = 'stock/location'
 
+    # Setup for Label printing
+    LABELNAME = 'location'
+    LABELITEM = 'locations'
+
     def getStockItems(self, **kwargs):
         return StockItem.list(self._api, location=self.pk, **kwargs)
 
@@ -34,11 +39,15 @@ class StockLocation(inventree.base.MetadataMixin, inventree.base.InventreeObject
         return StockLocation.list(self._api, parent=self.pk, **kwargs)
 
 
-class StockItem(inventree.base.BulkDeleteMixin, inventree.base.MetadataMixin, inventree.base.InventreeObject):
+class StockItem(inventree.base.BarcodeMixin, inventree.base.BulkDeleteMixin, inventree.base.MetadataMixin, inventree.label.LabelPrintingMixing, inventree.base.InventreeObject):
     """Class representing the StockItem database model."""
 
     URL = 'stock'
 
+    # Setup for Label printing
+    LABELNAME = 'stock'
+    LABELITEM = 'items'
+
     @classmethod
     def adjustStockItems(cls, api: inventree.api.InvenTreeAPI, method: str, items: list, **kwargs):
         """Perform a generic stock 'adjustment' action.
