@@ -12,6 +12,7 @@ from test_api import InvenTreeTestCase  # noqa: E402
 from inventree import part  # noqa: E402
 from inventree import order  # noqa: E402
 from inventree import company  # noqa: E402
+from inventree import stock  # noqa: E402
 
 
 class POTest(InvenTreeTestCase):
@@ -160,6 +161,80 @@ class POTest(InvenTreeTestCase):
         # Now there should be 0 lines left
         self.assertEqual(len(po.getExtraLineItems()), 0)
 
+    def test_order_cancel(self):
+        """Test that we can cancel a PurchaseOrder via the API"""
+        
+        n = len(order.PurchaseOrder.list(self.api)) + 1
+        ref = f"PO-{n}"
+
+        # Create a new PO
+        po = order.PurchaseOrder.create(self.api, data={
+            'supplier': 1,
+            'reference': ref,
+            'description': 'Some new order'
+        })
+
+        self.assertEqual(po.status, 10)
+        self.assertEqual(po.status_text, "Pending")
+
+        # Cancel the order
+        po.cancel()
+
+        self.assertEqual(po.status, 40)
+        self.assertEqual(po.status_text, "Cancelled")
+
+    def test_order_complete(self):
+        """Test that we can complete an order via the API"""
+
+        n = len(order.PurchaseOrder.list(self.api)) + 1
+        ref = f"PO-{n}"
+
+        # First, let's create a new PurchaseOrder
+        po = order.PurchaseOrder.create(self.api, data={
+            'supplier': 1,
+            'reference': ref,
+            'description': 'A new purchase order',
+        })
+
+        # Add some line items
+        for p in company.SupplierPart.list(self.api, supplier=1, limit=5):
+
+            po.addLineItem(
+                part=p.pk,
+                quantity=10,
+            )
+
+        # Check that lines have actually been added
+        lines = po.getLineItems()
+        self.assertTrue(len(lines) > 0)
+
+        # Initial status code is "PENDING"
+        self.assertEqual(po.status, 10)
+        self.assertEqual(po.status_text, "Pending")
+
+        # Issue the order
+        po.issue()
+        po.reload()
+
+        self.assertEqual(po.status, 20)
+        self.assertEqual(po.status_text, "Placed")
+
+        # Try to complete the order (should fail, as lines have not been received)
+        with self.assertRaises(HTTPError):
+            po.complete()
+    
+        po.reload()
+
+        # Check that order status has *not* changed
+        self.assertEqual(po.status, 20)
+
+        # Now, try to complete the order again, accepting incomplete
+        po.complete(accept_incomplete=True)
+        po.reload()
+
+        # Check that the order is now complete
+        self.assertEqual(po.status, 30)
+
     def test_purchase_order_delete(self):
         """
         Test that we can delete existing purchase orders
@@ -351,21 +426,36 @@ class SOTest(InvenTreeTestCase):
         self.assertEqual(len(attachments), n + 1)
 
     def test_so_shipment(self):
-        """
-        Test shipment functionality for a SalesOrder
-        """
+        """Test shipment functionality for a SalesOrder."""
 
-        # Grab the last available SalesOrder - should not have a shipment yet
-        orders = order.SalesOrder.list(self.api)
+        # Construct a new SalesOrder instance
+        so = order.SalesOrder.create(self.api, {
+            'customer': 4,
+            "description": "Selling some stuff",
+        })
 
-        if len(orders) > 0:
-            so = orders[-1]
-        else:
-            so = order.SalesOrder.create(self.api, {
-                'customer': 4,
-                'reference': "My new sales order",
-                "description": "Selling some stuff",
-            })
+        # Add some line items to the SalesOrder
+        for p in part.Part.list(self.api, is_template=False, salable=True, limit=5):
+            
+            # Create a line item matching the part
+            order.SalesOrderLineItem.create(
+                self.api,
+                data={
+                    'part': p.pk,
+                    'order': so.pk,
+                    'quantity': 10,
+                }
+            )
+
+            # Ensure there is available stock
+            stock.StockItem.create(
+                self.api,
+                data={
+                    'part': p.pk,
+                    'quantity': 25,
+                    'location': 1,
+                }
+            )
 
         # The shipments list should return something which is not none
         self.assertIsNotNone(so.getShipments())
@@ -421,6 +511,7 @@ class SOTest(InvenTreeTestCase):
         # (which should be overwritten)
         notes = f'Test shipment number {num_shipments+1} for order {so.pk}'
         tracking_number = '93414134343'
+
         shipment_2 = so.addShipment(
             reference=f'Package {num_shipments+1}',
             order=10103413,
@@ -450,13 +541,17 @@ class SOTest(InvenTreeTestCase):
         
         # Assign each line item to this shipment
         for si in so.getLineItems():
-            response = si.allocateToShipment(shipment_2)
-            # Remember what we are doing for later check
-            # a response of None means nothing was allocated
-            if response is not None:
-                allocated_quantities[si.pk] = (
-                    {x['stock_item']: float(x['quantity']) for x in response['items']}
-                )
+            # If there is no stock available, delete this line
+            if si.available_stock == 0:
+                si.delete()
+            else:
+                response = si.allocateToShipment(shipment_2)
+                # Remember what we are doing for later check
+                # a response of None means nothing was allocated
+                if response is not None:
+                    allocated_quantities[si.pk] = (
+                        {x['stock_item']: float(x['quantity']) for x in response['items']}
+                    )
 
         # Check saved values
         for so_part in so.getLineItems():
@@ -467,8 +562,35 @@ class SOTest(InvenTreeTestCase):
                         allocated_quantities[so_part.pk]
                     )
 
-        # Complete the shipment, with minimum information
+        # Attempt to complete the shipment, but no items have been allocated
         shipment_2.complete()
 
         # Make sure date is not None
         self.assertIsNotNone(shipment_2.shipment_date)
+
+        # Try to complete this order
+        # Ship remaining shipments first
+        for shp in so.getShipments():
+            # Delete shipment if it has no allocations
+            if len(shp.allocations) == 0:
+                shp.delete()
+                continue
+            # If the shipment has no date, try to mark it shipped
+            if shp.shipment_date is None:
+                shp.ship()
+        so.complete()
+        self.assertEqual(so.status, 20)
+        self.assertEqual(so.status_text, 'Shipped')
+
+    def test_order_cancel(self):
+        """Test cancel sales order"""
+        
+        so = order.SalesOrder.create(self.api, {
+            'customer': 4,
+            "description": "Selling some more stuff",
+        })
+
+        so.cancel()
+
+        self.assertEqual(so.status, 40)
+        self.assertEqual(so.status_text, 'Cancelled')
