@@ -603,3 +603,57 @@ class PartTest(InvenTreeTestCase):
         self.assertEqual(len(metadata.keys()), 2)
         self.assertEqual(metadata['foo'], 'rab')
         self.assertEqual(metadata['hello'], 'world')
+
+
+class PartBarcodeTest(InvenTreeTestCase):
+    """Tests for Part barcode functionality"""
+
+    def test_barcode_assign(self):
+        """Tests for assigning barcodes to Part instances"""
+
+        barcode = 'ABCD-1234-XYZ'
+
+        # Grab a part from the database
+        part_1 = Part(self.api, pk=1)
+
+        # First ensure that there is *no* barcode assigned to this item
+        part_1.unassignBarcode()
+
+        # Assign a barcode to this part (should auto-reload)
+        response = part_1.assignBarcode(barcode)
+
+        self.assertEqual(response['success'], 'Assigned barcode to part instance')
+        self.assertEqual(response['barcode_data'], barcode)
+        
+        # Attempt to assign the same barcode to a different part (should error)
+        part_2 = Part(self.api, pk=2)
+
+        # Ensure this part does not have an associated barcode
+        part_2.unassignBarcode()
+
+        with self.assertRaises(HTTPError):
+            response = part_2.assignBarcode(barcode)
+
+        # Scan the barcode (should point back to part_1)
+        response = self.api.scanBarcode(barcode)
+
+        self.assertEqual(response['barcode_data'], barcode)
+        self.assertEqual(response['part']['pk'], 1)
+
+        # Unassign from part_1
+        part_1.unassignBarcode()
+
+        # Now assign to part_2
+        response = part_2.assignBarcode(barcode)
+        self.assertEqual(response['barcode_data'], barcode)
+    
+        # Scan again
+        response = self.api.scanBarcode(barcode)
+        self.assertEqual(response['part']['pk'], 2)
+
+        # Unassign from part_2
+        part_2.unassignBarcode()
+
+        # Scanning this time should yield no results
+        with self.assertRaises(HTTPError):
+            response = self.api.scanBarcode(barcode)
